@@ -1,5 +1,5 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
-/* stream.c - provides a STREAM object
+/* -*- Mode: C; c-file-style: "bsd" -*-
+ * stream.c - provides a STREAM object
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with OpenCDK; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,9 +39,9 @@
 #include "types.h"
 
 
-static int stream_flush( CDK_STREAM s );
-static int stream_filter_write( CDK_STREAM s );
-static int stream_cache_flush( CDK_STREAM s, FILE * fp );
+static int stream_flush( cdk_stream_t s );
+static int stream_filter_write( cdk_stream_t s );
+static int stream_cache_flush( cdk_stream_t s, FILE * fp );
 
 
 /**
@@ -49,16 +49,16 @@ static int stream_cache_flush( CDK_STREAM s, FILE * fp );
  * @file: The file to open
  * @ret_s: The new STREAM object
  **/
-int
-cdk_stream_open( const char * file, CDK_STREAM * ret_s )
+cdk_error_t
+cdk_stream_open( const char * file, cdk_stream_t * ret_s )
 {
-    CDK_STREAM s;
+    cdk_stream_t s;
 
     if( !file || !ret_s )
         return CDK_Inv_Value;
 
     _cdk_log_debug( "open stream `%s'\n", file );
-  
+    *ret_s = NULL;
     s = cdk_calloc( 1, sizeof *s );
     if( !s )
         return CDK_Out_Of_Core;
@@ -84,35 +84,28 @@ cdk_stream_open( const char * file, CDK_STREAM * ret_s )
  * @file: The name of the new file
  * @ret_s: The new STREAM object
  **/
-int
-cdk_stream_new( const char * file, CDK_STREAM * ret_s )
+cdk_error_t
+cdk_stream_new( const char * file, cdk_stream_t * ret_s )
 {
-    CDK_STREAM s;
-
-    _cdk_log_debug( "new stream `%s'\n", file? file : "[temp]" );
+    cdk_stream_t s;
 
     if( !ret_s )
         return CDK_Inv_Value;
-    
+
+    _cdk_log_debug( "new stream `%s'\n", file? file : "[temp]" );
+    *ret_s = NULL;
     s = cdk_calloc( 1, sizeof *s );
     if( !s )
-        return CDK_Out_Of_Core;
-  
+        return CDK_Out_Of_Core;  
     s->flags.write = 1;
-    if( !file ) {
+    if( !file )
         s->flags.temp = 1;
-        s->fp = tmpfile( );
-        if( !s->fp ) {
+    else {
+        s->fname = cdk_strdup( file );
+        if( !s->fname ) {
             cdk_free( s );
-            return CDK_File_Error;
+            return CDK_Out_Of_Core;
         }
-        *ret_s = s;
-        return 0;
-    }
-    s->fname = cdk_strdup( file );
-    if( !s->fname ) {
-        cdk_free( s );
-        return CDK_Out_Of_Core;
     }
     s->fp = tmpfile( );
     if( !s->fp ) {
@@ -133,14 +126,16 @@ cdk_stream_new( const char * file, CDK_STREAM * ret_s )
  * The difference to cdk_stream_new is, that no filtering can be used with
  * this kind of stream and everything is written directly to the stream.
  **/
-int
-cdk_stream_create( const char * file, CDK_STREAM * ret_s )
+cdk_error_t
+cdk_stream_create( const char * file, cdk_stream_t * ret_s )
 {
-    CDK_STREAM s;
+    cdk_stream_t s;
 
     if( !file || !ret_s )
         return CDK_Inv_Value;
+    
     _cdk_log_debug( "create stream `%s'\n", file );
+    *ret_s = NULL;
     s = cdk_calloc( 1, sizeof * s );
     if( !s )
         return CDK_Out_Of_Core;
@@ -162,26 +157,24 @@ cdk_stream_create( const char * file, CDK_STREAM * ret_s )
 }
     
 
-CDK_STREAM
+cdk_stream_t
 cdk_stream_tmp( void )
 {
-    CDK_STREAM s;
-    int rc;
-  
-    rc = cdk_stream_new( NULL, &s );
+    cdk_stream_t s;
+    int rc = cdk_stream_new( NULL, &s );
     if( !rc )
         return s;
     return NULL;
 }
 
 
-CDK_STREAM
+cdk_stream_t
 cdk_stream_tmp_from_mem( const void * buf, size_t count )
 {
-    CDK_STREAM s;
+    cdk_stream_t s;
     int nwritten;
 
-    s = cdk_stream_tmp(  );
+    s = cdk_stream_tmp( );
     if( !s )
         return NULL;
   
@@ -196,10 +189,10 @@ cdk_stream_tmp_from_mem( const void * buf, size_t count )
 
   
 
-CDK_STREAM
-cdk_stream_fpopen( FILE * fp, unsigned write_mode )
+cdk_stream_t
+_cdk_stream_fpopen( FILE * fp, unsigned write_mode )
 {
-  CDK_STREAM s;
+  cdk_stream_t s;
 
   s = cdk_calloc( 1, sizeof *s );
   if( !s )
@@ -213,10 +206,10 @@ cdk_stream_fpopen( FILE * fp, unsigned write_mode )
 }
 
 
-int
-_cdk_stream_append( const char * file, CDK_STREAM * ret_s )
+cdk_error_t
+_cdk_stream_append( const char * file, cdk_stream_t * ret_s )
 {
-    CDK_STREAM s;
+    cdk_stream_t s;
     FILE * fp;
     int rc;
 
@@ -224,7 +217,7 @@ _cdk_stream_append( const char * file, CDK_STREAM * ret_s )
         return CDK_Inv_Value;
     rc = cdk_stream_open( file, &s );
     if( rc )
-        return rc;    
+        return rc;
     fp = fopen( file, "a+b" );
     if( !fp ) {
         cdk_stream_close( s );
@@ -239,22 +232,28 @@ _cdk_stream_append( const char * file, CDK_STREAM * ret_s )
 
       
 int
-cdk_stream_control( CDK_STREAM s, int ctl, int val )
+cdk_stream_control( cdk_stream_t s, int ctl, int val )
 {
     if( !s )
         return CDK_Inv_Value;
-    
+
+    if( val == -1 ) {
+        switch( ctl ) {
+        case CDK_STREAMCTL_COMPRESSED: return s->flags.compressed;
+        }
+        return 0;
+    }
     switch( ctl ) {
-    case CDK_STREAMCTL_RESET  : s->flags.reset = val; break;
     case CDK_STREAMCTL_DISABLE: s->flags.no_filter = val; break;
-    default                   : return CDK_Inv_Value;
+    case CDK_STREAMCTL_COMPRESSED: s->flags.compressed = val; break;
+    default                   : return CDK_Inv_Mode;
     }
     return 0;
 }
 
 
-int
-cdk_stream_flush( CDK_STREAM s )
+cdk_error_t
+cdk_stream_flush( cdk_stream_t s )
 {
     int rc = 0;
   
@@ -279,7 +278,7 @@ cdk_stream_flush( CDK_STREAM s )
 
 
 void
-cdk_stream_tmp_set_mode( CDK_STREAM s, int val )
+cdk_stream_tmp_set_mode( cdk_stream_t s, int val )
 {
     if( s && s->flags.temp )
         s->fmode = val;
@@ -296,8 +295,8 @@ cdk_stream_tmp_set_mode( CDK_STREAM s, int val )
  * But for the case it's a write stream, we need to apply all registered
  * filters now. The file is closed in the filter function and not here.
  **/
-int
-cdk_stream_close( CDK_STREAM s )
+cdk_error_t
+cdk_stream_close( cdk_stream_t s )
 {
     struct stream_filter_s * f, * f2;
     int rc = 0;
@@ -318,8 +317,8 @@ cdk_stream_close( CDK_STREAM s )
     f = s->filters;
     while( f ) {
         f2 = f->next;
-        if( f->fnc )
-            f->fnc( f->opaque, STREAMCTL_FREE, NULL, NULL );
+        if( f->fnct )
+            f->fnct( f->opaque, STREAMCTL_FREE, NULL, NULL );
         cdk_free( f );
         f = f2;
     }
@@ -339,28 +338,28 @@ cdk_stream_close( CDK_STREAM s )
  * This function will only work with read streams.
  **/
 int
-cdk_stream_eof( CDK_STREAM s )
+cdk_stream_eof( cdk_stream_t s )
 {
     return s? s->flags.eof : -1;
 }
 
 
 const char *
-_cdk_stream_get_fname( CDK_STREAM s )
+_cdk_stream_get_fname( cdk_stream_t s )
 {
     return s? s->fname : NULL;
 }
 
 
 FILE *
-_cdk_stream_get_fp( CDK_STREAM s )
+_cdk_stream_get_fp( cdk_stream_t s )
 {
     return s? s->fp : NULL;
 }
 
 
 int
-_cdk_stream_get_errno( CDK_STREAM s )
+_cdk_stream_get_errno( cdk_stream_t s )
 {
     return s? s->error : CDK_Inv_Value;
 }
@@ -374,7 +373,7 @@ _cdk_stream_get_errno( CDK_STREAM s )
  * write stream is not flushed or even no data was inserted.
  **/
 unsigned
-cdk_stream_get_length( CDK_STREAM s )
+cdk_stream_get_length( cdk_stream_t s )
 {
     struct stat statbuf;
     int rc;
@@ -396,7 +395,7 @@ cdk_stream_get_length( CDK_STREAM s )
 
 
 static struct stream_filter_s *
-filter_add( CDK_STREAM s )
+filter_add2( cdk_stream_t s )
 {
     struct stream_filter_s * f;
 
@@ -412,14 +411,14 @@ filter_add( CDK_STREAM s )
 
 
 static struct stream_filter_s *
-filter_search( CDK_STREAM s, FILTER_FUNC fnc )
+filter_search( cdk_stream_t s, filter_fnct_t fnc )
 {
     struct stream_filter_s * f;
 
     assert( s );
     
     for( f = s->filters; f; f = f->next ) {
-        if( f->fnc == fnc )
+        if( f->fnct == fnc )
             return f;
     }
     return NULL;
@@ -427,37 +426,38 @@ filter_search( CDK_STREAM s, FILTER_FUNC fnc )
 
 
 struct stream_filter_s *
-stream_filter_add( CDK_STREAM s, FILTER_FUNC fnc, int type)
+filter_add( cdk_stream_t s, filter_fnct_t fnc, int type )
 {
-    struct stream_filter_s * flt;
+    struct stream_filter_s * f;
 
+    assert( s );
+    
     s->flags.filtrated = 0;
-    flt = filter_search( s, fnc );
-    if( flt )
-        return flt;
-  
-    flt = filter_add( s );
-    if( !flt )
+    f = filter_search( s, fnc );
+    if( f )
+        return f;
+    f = filter_add2( s );
+    if( !f )
         return NULL;
-    flt->fnc = fnc;
-    flt->flags.enabled = 1;
-    flt->tmp = NULL;
-    flt->type = type;
+    f->fnct = fnc;
+    f->flags.enabled = 1;
+    f->tmp = NULL;
+    f->type = type;
     switch( type ) {
-    case fARMOR    : flt->opaque = &flt->u.afx; break;
-    case fCIPHER   : flt->opaque = &flt->u.cfx; break;
-    case fPLAINTEXT: flt->opaque = &flt->u.pfx; break;
-    case fCOMPRESS : flt->opaque = &flt->u.zfx; break;
-    case fHASH     : flt->opaque = &flt->u.mfx; break;
-    case fTEXT     : flt->opaque = &flt->u.tfx; break;
-    default        : flt->opaque = NULL;
+    case fARMOR    : f->opaque = &f->u.afx; break;
+    case fCIPHER   : f->opaque = &f->u.cfx; break;
+    case fLITERAL: f->opaque = &f->u.pfx; break;
+    case fCOMPRESS : f->opaque = &f->u.zfx; break;
+    case fHASH     : f->opaque = &f->u.mfx; break;
+    case fTEXT     : f->opaque = &f->u.tfx; break;
+    default        : f->opaque = NULL;
     }
-    return flt;
+    return f;
 }
 
 
 static int
-stream_get_mode( CDK_STREAM s )
+stream_get_mode( cdk_stream_t s )
 {
     assert( s );
     
@@ -467,12 +467,12 @@ stream_get_mode( CDK_STREAM s )
 }
 
 
-static FILTER_FUNC
+static filter_fnct_t
 stream_id_to_filter( int type )
 {
     switch( type ) {
     case fARMOR    : return _cdk_filter_armor;
-    case fPLAINTEXT: return _cdk_filter_plaintext;
+    case fLITERAL: return _cdk_filter_literal;
     case fTEXT     : return _cdk_filter_text;
     case fCIPHER   : return _cdk_filter_cipher;
     case fCOMPRESS : return _cdk_filter_compress;
@@ -487,11 +487,11 @@ stream_id_to_filter( int type )
  * @type: The numberic filter ID.
  *
  **/
-int
-cdk_stream_filter_disable( CDK_STREAM s, int type )
+cdk_error_t
+cdk_stream_filter_disable( cdk_stream_t s, int type )
 {
     struct stream_filter_s * f;
-    FILTER_FUNC fnc;
+    filter_fnct_t fnc;
 
     if( !s )
         return CDK_Inv_Value;
@@ -504,7 +504,7 @@ cdk_stream_filter_disable( CDK_STREAM s, int type )
 
 
 static int
-stream_fp_replace( CDK_STREAM s, FILE ** tmp )
+stream_fp_replace( cdk_stream_t s, FILE ** tmp )
 {
     int rc;
 
@@ -523,7 +523,7 @@ stream_fp_replace( CDK_STREAM s, FILE ** tmp )
    use tmpfile () all the time. That's why we open the real file when there
    is no last filter. */
 static int
-stream_filter_write( CDK_STREAM s )
+stream_filter_write( cdk_stream_t s )
 {
     struct stream_filter_s * f;
     int rc = 0;
@@ -555,7 +555,7 @@ stream_filter_write( CDK_STREAM s )
             if( rc )
                 break;
         }
-        rc = f->fnc( f->opaque, f->ctl, s->fp, f->tmp );
+        rc = f->fnct( f->opaque, f->ctl, s->fp, f->tmp );
         _cdk_log_debug( "filter [write]: type=%d rc=%d\n", f->type, rc );
         if( !rc )
             rc = stream_fp_replace( s, &f->tmp );
@@ -576,7 +576,7 @@ stream_filter_write( CDK_STREAM s )
    original file handle will be closed and replace with the temp handle.
    The file pointer will be set to the begin and the game starts again. */
 static int
-stream_filter_read( CDK_STREAM s )
+stream_filter_read( cdk_stream_t s )
 {
     struct stream_filter_s * f;
     int rc = 0;
@@ -594,7 +594,7 @@ stream_filter_read( CDK_STREAM s )
             rc = CDK_File_Error;
             break;
         }
-        rc = f->fnc( f->opaque, f->ctl, s->fp, f->tmp );
+        rc = f->fnct( f->opaque, f->ctl, s->fp, f->tmp );
         _cdk_log_debug( "filter %s [read]: type=%d rc=%d\n",
                         s->fname? s->fname : "[temp]", f->type, rc );
         if( rc )
@@ -614,15 +614,19 @@ stream_filter_read( CDK_STREAM s )
         rc = cdk_stream_seek( s, 0 );
         if( rc )
             break;
-        if( s->flags.reset )
-            f->flags.enabled = 0;
+        /* Disable the filter after it was successfully used. The idea
+           is the following: let's say the armor filter was pushed and
+           later more filters were added. The second time the filter code
+           will be executed, only the new filter should be started but
+           not the old because we already used it. */
+        f->flags.enabled = 0;
     }
     return rc;
 }
 
 
 void *
-_cdk_stream_get_opaque( CDK_STREAM s, int id )
+_cdk_stream_get_opaque( cdk_stream_t s, int fid )
 {
     struct stream_filter_s * f;
 
@@ -630,7 +634,7 @@ _cdk_stream_get_opaque( CDK_STREAM s, int id )
         return NULL;
     
     for( f = s->filters; f; f = f->next ) {
-        if( f->type == id )
+        if( f->type == fid )
             return f->opaque;
     }
     return NULL;
@@ -648,7 +652,7 @@ _cdk_stream_get_opaque( CDK_STREAM s, int id )
  * need to add the filters in reserved order.
  **/
 int
-cdk_stream_read( CDK_STREAM s, void * buf, size_t count )
+cdk_stream_read( cdk_stream_t s, void * buf, size_t count )
 {
     int nread;
     int rc;
@@ -679,7 +683,7 @@ cdk_stream_read( CDK_STREAM s, void * buf, size_t count )
 
       
 int
-cdk_stream_getc( CDK_STREAM s )
+cdk_stream_getc( cdk_stream_t s )
 {
     unsigned char buf[2];
     int nread;
@@ -707,7 +711,7 @@ cdk_stream_getc( CDK_STREAM s )
  * partial flushing.
  **/
 int
-cdk_stream_write( CDK_STREAM s, const void * buf, size_t count )
+cdk_stream_write( cdk_stream_t s, const void * buf, size_t count )
 {
     int nwritten;
 
@@ -718,7 +722,7 @@ cdk_stream_write( CDK_STREAM s, const void * buf, size_t count )
         return CDK_Inv_Mode; /* this is a read stream */
 
     if( !buf && !count )
-        return stream_flush( s  );
+        return stream_flush( s );
 
     if( s->cache.on ) {
         if( s->cache.size + count > sizeof( s->cache.buf ) )
@@ -736,7 +740,7 @@ cdk_stream_write( CDK_STREAM s, const void * buf, size_t count )
 
 
 int
-cdk_stream_putc( CDK_STREAM s, int c )
+cdk_stream_putc( cdk_stream_t s, int c )
 {
     unsigned char buf[2];
     int nwritten;
@@ -754,14 +758,14 @@ cdk_stream_putc( CDK_STREAM s, int c )
 
 
 long
-cdk_stream_tell( CDK_STREAM s )
+cdk_stream_tell( cdk_stream_t s )
 {
     return s? ftell( s->fp ): (long)-1;
 }
 
 
-int
-cdk_stream_seek( CDK_STREAM s, long offset )
+cdk_error_t
+cdk_stream_seek( cdk_stream_t s, long offset )
 {
     int rc;
 
@@ -777,7 +781,7 @@ cdk_stream_seek( CDK_STREAM s, long offset )
 
 
 static int
-stream_flush( CDK_STREAM s )
+stream_flush( cdk_stream_t s )
 {
     int rc;
     
@@ -790,119 +794,119 @@ stream_flush( CDK_STREAM s )
 }
 
 
-int
-cdk_stream_set_armor_flag( CDK_STREAM s, int type )
+cdk_error_t
+cdk_stream_set_armor_flag( cdk_stream_t s, int type )
 {
-    struct stream_filter_s * flt;
+    struct stream_filter_s * f;
 
     if( !s )
         return CDK_Inv_Value;
-    flt = stream_filter_add( s, _cdk_filter_armor, fARMOR );
-    if( !flt )
+    f = filter_add( s, _cdk_filter_armor, fARMOR );
+    if( !f )
         return CDK_Out_Of_Core;
-    flt->u.afx.idx = flt->u.afx.idx2 = type;
-    flt->ctl = stream_get_mode( s );
+    f->u.afx.idx = f->u.afx.idx2 = type;
+    f->ctl = stream_get_mode( s );
     return 0;
 }
 
 
-int
-cdk_stream_set_literal_flag( CDK_STREAM s, int mode, const char * fname )
+cdk_error_t
+cdk_stream_set_literal_flag( cdk_stream_t s, int mode, const char * fname )
 {
-    struct stream_filter_s * flt;
+    struct stream_filter_s * f;
 
     if( !s )
         return CDK_Inv_Value;
-    flt = stream_filter_add( s, _cdk_filter_plaintext, fPLAINTEXT );
-    if( !flt )
+    f = filter_add( s, _cdk_filter_literal, fLITERAL );
+    if( !f )
         return CDK_Out_Of_Core;
-    flt->u.pfx.mode = mode;
-    flt->u.pfx.filename = fname? cdk_strdup( fname ) : NULL;
-    flt->ctl = stream_get_mode( s );
+    f->u.pfx.mode = mode;
+    f->u.pfx.filename = fname? cdk_strdup( fname ) : NULL;
+    f->ctl = stream_get_mode( s );
     if( s->blkmode ) {
-        flt->u.pfx.blkmode.on = 1;
-        flt->u.pfx.blkmode.size = s->blkmode;
+        f->u.pfx.blkmode.on = 1;
+        f->u.pfx.blkmode.size = s->blkmode;
     }
     return 0;
 }
 
 
-int
-cdk_stream_set_cipher_flag( CDK_STREAM s, CDK_DEK dek, int use_mdc )
+cdk_error_t
+cdk_stream_set_cipher_flag( cdk_stream_t s, cdk_dek_t dek, int use_mdc )
 {
-    struct stream_filter_s * flt;
+    struct stream_filter_s * f;
 
     if( !s )
         return CDK_Inv_Value;
-    flt = stream_filter_add( s, _cdk_filter_cipher, fCIPHER );
-    if( !flt )
+    f = filter_add( s, _cdk_filter_cipher, fCIPHER );
+    if( !f )
         return CDK_Out_Of_Core;
     dek->use_mdc = use_mdc;
-    flt->ctl = stream_get_mode( s );
-    flt->u.cfx.dek = dek;
-    flt->u.cfx.mdc_method = use_mdc? GCRY_MD_SHA1 : 0;
+    f->ctl = stream_get_mode( s );
+    f->u.cfx.dek = dek;
+    f->u.cfx.mdc_method = use_mdc? GCRY_MD_SHA1 : 0;
     if( s->blkmode ) {
-        flt->u.cfx.blkmode.on = 1;
-        flt->u.cfx.blkmode.size = s->blkmode;
+        f->u.cfx.blkmode.on = 1;
+        f->u.cfx.blkmode.size = s->blkmode;
     }
     return 0;
 }
 
 
-int
-cdk_stream_set_compress_flag( CDK_STREAM s, int algo, int level )
+cdk_error_t
+cdk_stream_set_compress_flag( cdk_stream_t s, int algo, int level )
 {
-    struct stream_filter_s * flt;
+    struct stream_filter_s * f;
 
     if( !s )
         return CDK_Inv_Value;
-    flt = stream_filter_add( s, _cdk_filter_compress, fCOMPRESS );
-    if( !flt )
+    f = filter_add( s, _cdk_filter_compress, fCOMPRESS );
+    if( !f )
         return CDK_Out_Of_Core;
-    flt->ctl = stream_get_mode( s );
-    flt->u.zfx.algo = algo;
-    flt->u.zfx.level = level;
+    f->ctl = stream_get_mode( s );
+    f->u.zfx.algo = algo;
+    f->u.zfx.level = level;
     return 0;
 }
 
 
-int
-cdk_stream_set_text_flag( CDK_STREAM s, const char * lf )
+cdk_error_t
+cdk_stream_set_text_flag( cdk_stream_t s, const char * lf )
 {
-    struct stream_filter_s * flt;
+    struct stream_filter_s * f;
 
     if( !s )
         return CDK_Inv_Value;
-    flt = stream_filter_add( s, _cdk_filter_text, fTEXT );
-    if( !flt )
+    f = filter_add( s, _cdk_filter_text, fTEXT );
+    if( !f )
         return CDK_Out_Of_Core;
-    flt->ctl = stream_get_mode( s );
-    flt->u.tfx.lf = lf;
+    f->ctl = stream_get_mode( s );
+    f->u.tfx.lf = lf;
     return 0;
 }
 
 
-int
-cdk_stream_set_hash_flag( CDK_STREAM s, int algo )
+cdk_error_t
+cdk_stream_set_hash_flag( cdk_stream_t s, int algo )
 {
-    struct stream_filter_s * flt;
+    struct stream_filter_s * f;
 
     if( !s )
         return CDK_Inv_Value;    
     if( stream_get_mode( s ) )
         return CDK_Inv_Mode;
-    flt = stream_filter_add( s, _cdk_filter_hash, fHASH );
-    if( !flt)
+    f = filter_add( s, _cdk_filter_hash, fHASH );
+    if( !f )
         return CDK_Out_Of_Core;
-    flt->ctl = stream_get_mode( s );
-    flt->u.mfx.digest_algo = algo;
-    flt->flags.rdonly = 1;
+    f->ctl = stream_get_mode( s );
+    f->u.mfx.digest_algo = algo;
+    f->flags.rdonly = 1;
     return 0;
 }
 
 
-int
-cdk_stream_set_cache( CDK_STREAM s, int val )
+cdk_error_t
+cdk_stream_set_cache( cdk_stream_t s, int val )
 {
     if( !s )
         return CDK_Inv_Value;
@@ -914,7 +918,7 @@ cdk_stream_set_cache( CDK_STREAM s, int val )
 
 
 static int
-stream_cache_flush( CDK_STREAM s, FILE * fp )
+stream_cache_flush( cdk_stream_t s, FILE * fp )
 {
     int nwritten;
 
@@ -932,8 +936,8 @@ stream_cache_flush( CDK_STREAM s, FILE * fp )
 }
 
 
-int
-cdk_stream_kick_off( CDK_STREAM inp, CDK_STREAM out )
+cdk_error_t
+cdk_stream_kick_off( cdk_stream_t inp, cdk_stream_t out )
 {
     byte buf[8192];
     int nread, nwritten;
@@ -963,12 +967,13 @@ cdk_stream_kick_off( CDK_STREAM inp, CDK_STREAM out )
  * Map the data of the given stream into a memory section. @ret_count
  * contains the length of the buffer.
  **/
-int
-cdk_stream_mmap( CDK_STREAM s, byte ** ret_buf, size_t * ret_count )
+cdk_error_t
+cdk_stream_mmap( cdk_stream_t s, byte ** ret_buf, size_t * ret_count )
 {
     const u32 max_filesize = 16777216;
     u32 len, oldpos;
-    int n;
+    int n, rc;
+    char * p;
 
     if( !s || !ret_buf || !ret_count )
         return CDK_Inv_Value;
@@ -976,25 +981,54 @@ cdk_stream_mmap( CDK_STREAM s, byte ** ret_buf, size_t * ret_count )
     *ret_count = 0;
     *ret_buf = NULL;
     oldpos = cdk_stream_tell( s );
-    cdk_stream_flush( s );
-    cdk_stream_seek( s, 0 );
+    rc = cdk_stream_flush( s );
+    if( !rc )
+        rc = cdk_stream_seek( s, 0 );
+    if( rc )
+        return rc;
     len = cdk_stream_get_length( s );
     if( !len || len > max_filesize )
         return 0;
-    *ret_buf = cdk_calloc( 1, len+1 );
-    if( !*ret_buf )
+    p = *ret_buf = cdk_calloc( 1, len+1 );
+    if( !p )
         return 0;
     *ret_count = len;
-    n = cdk_stream_read( s, *ret_buf, len );
+    n = cdk_stream_read( s, p, len );
     if( n != len )
         *ret_count = n;
-    cdk_stream_seek( s, oldpos );
-    return 0;
+    rc = cdk_stream_seek( s, oldpos );
+    return rc;
 }
-        
+
+
+/**
+ * cdk_stream_peek:
+ * @inp: the input stream handle
+ * @s: buffer
+ * @count: number of bytes to peek
+ *
+ * The function acts like cdk_stream_read with the difference that
+ * the file pointer is moved to the old position after the bytes were read.
+ **/
+int
+cdk_stream_peek( cdk_stream_t inp, byte *s, size_t count )
+{
+    unsigned off;
+    int nbytes, rc;
+
+    if( !inp || !s )
+        return CDK_Inv_Value;
+    off = cdk_stream_tell( inp );
+    nbytes = _cdk_stream_gets( inp, s, count );
+    rc = cdk_stream_seek( inp, off );
+    if( rc )
+        return 0;
+    return nbytes;
+}
+
 
 int
-_cdk_stream_gets( CDK_STREAM s, char * buf, size_t count )
+_cdk_stream_gets( cdk_stream_t s, char * buf, size_t count )
 {
     int c, i = 0;
 
@@ -1014,14 +1048,14 @@ _cdk_stream_gets( CDK_STREAM s, char * buf, size_t count )
 
 
 int
-_cdk_stream_puts( CDK_STREAM s, const char * buf )
+_cdk_stream_puts( cdk_stream_t s, const char * buf )
 {
     return cdk_stream_write( s, buf, strlen( buf ) );
 }
 
 
 int
-_cdk_stream_set_blockmode( CDK_STREAM s, size_t nbytes )
+_cdk_stream_set_blockmode( cdk_stream_t s, size_t nbytes )
 {
     if( !s )
         return CDK_Inv_Value;
@@ -1032,9 +1066,8 @@ _cdk_stream_set_blockmode( CDK_STREAM s, size_t nbytes )
 
 
 int
-_cdk_stream_get_blockmode( CDK_STREAM s )
+_cdk_stream_get_blockmode( cdk_stream_t s )
 {
     return s? s->blkmode : 0;
 }
 
-    
